@@ -3,6 +3,7 @@ __all__ = [
     "TextExtractor",
     "EmbeddedExtractor",
     "MarkdownExtractor",
+    "MarkdownParserWithUncheckedLinks",
 ]
 
 
@@ -10,43 +11,44 @@ import re
 from dataclasses import replace
 from itertools import islice
 from pathlib import Path
-from typing import Dict, Iterable, Iterator, List, Mapping, Optional, Tuple, Union
-from urllib.parse import unquote, urlparse
+from typing import Any, Dict, Iterable, Iterator, List, Mapping, Optional, Tuple, Union
+from urllib.parse import urlparse
 
 from beet import Cache, DataPack, ResourcePack
 from beet.core.utils import FileSystemPath
 from markdown_it import MarkdownIt
-from markdown_it.common import normalize_url
 from markdown_it.token import Token
 
 from .directive import Directive
 from .fragment import Fragment
 
-# Patch markdown_it to allow arbitrary data urls
-# https://github.com/executablebooks/markdown-it-py/issues/128
-# TODO: Will soon be able to use custom validateLink
-normalize_url.GOOD_DATA_RE = re.compile(r"^data:")
-
 
 class Extractor:
     """Base class for extractors."""
 
     directives: Dict[str, Directive]
     regex: Optional["re.Pattern[str]"]
+    escaped_regex: Optional["re.Pattern[str]"]
     cache: Optional[Cache]
 
     def __init__(self, cache: Optional[Cache] = None):
         self.directives = {}
         self.regex = None
+        self.escaped_regex = None
         self.cache = cache
 
     def generate_regex(self) -> str:
         """Return a regex that can match the current directives."""
-        names = "|".join(name for name in self.directives)
+        names = "|".join(self.directives)
         modifier = r"(?:\((?P<modifier>[^)]*)\)|\b)"
         arguments = r"(?P<arguments>.*)"
         return f"@(?P<name>{names}){modifier}{arguments}"
 
+    def generate_escaped_regex(self) -> str:
+        """Return a regex that can match escaped fragments."""
+        names = "|".join(self.directives)
+        return fr"(@@+(?:{names})\b.*)"
+
     def compile_regex(self, regex: str) -> "re.Pattern[str]":
         """Return the compiled pattern for the directive regex."""
         return re.compile(f"^{regex}$", flags=re.MULTILINE)
@@ -58,6 +60,7 @@ class Extractor:
         if self.regex is None or self.directives != directives:
             self.directives = directives
             self.regex = self.compile_regex(self.generate_regex())
+            self.escaped_regex = self.compile_regex(self.generate_escaped_regex())
 
         return self.regex
 
@@ -143,6 +146,7 @@ class TextExtractor(Extractor):
         directives: Mapping[str, Directive],
     ) -> Iterator[Fragment]:
         tokens = self.get_regex(directives).split(source + "\n")
+        assert self.escaped_regex
 
         it = iter(tokens)
         newlines = next(it).count("\n")
@@ -154,6 +158,10 @@ class TextExtractor(Extractor):
                 return
             else:
                 content = content.partition("\n")[-1]
+                content = "".join(
+                    s.replace("@@", "@", 1) if i % 2 else s
+                    for i, s in enumerate(self.escaped_regex.split(content))
+                )
                 yield Fragment(
                     start_line=newlines,
                     end_line=(newlines := newlines + content.count("\n") + 1),
@@ -171,6 +179,22 @@ class EmbeddedExtractor(TextExtractor):
     def generate_regex(self) -> str:
         return r"(?://|#)\s*" + super().generate_regex()
 
+    def generate_escaped_regex(self) -> str:
+        return r"(?://|#)\s*" + super().generate_escaped_regex()
+
+
+class MarkdownParserWithUncheckedLinks(MarkdownIt):
+    """MarkownIt subclass that removes link checks because lectern has its own logic."""
+
+    def normalizeLink(self, url: str) -> str:
+        return url
+
+    def normalizeLinkText(self, link: str) -> str:
+        return link
+
+    def validateLink(self, url: str) -> bool:
+        return True
+
 
 class MarkdownExtractor(Extractor):
     """Extractor for markdown files."""
@@ -184,7 +208,7 @@ class MarkdownExtractor(Extractor):
         super().__init__(cache)
         self.embedded_extractor = EmbeddedExtractor(cache)
         self.comment_extractor = TextExtractor(cache)
-        self.parser = MarkdownIt()
+        self.parser = MarkdownParserWithUncheckedLinks()
         self.html_comment_regex = re.compile(r"\s*<!--\s*(.+?)\s*-->\s*")
 
     def extract(
@@ -515,11 +539,11 @@ class MarkdownExtractor(Extractor):
         start_line: int,
         end_line: int,
         match: "re.Match[str]",
-        link: str,
+        link: Any,
         external_files: Optional[FileSystemPath] = None,
     ) -> Fragment:
         """Helper for creating a fragment from a link."""
-        url = unquote(link)  # TODO: Will soon be able to use custom normalizeLink
+        url = str(link)
         path = None
 
         if urlparse(url).path == url:
